﻿using System;
using System.Threading;
using HeuristicLab.Optimization;

namespace HeuristicLab.Problems.MetaOptimization {

  public static class AlgorithmExtensions {
    public static void StartSync(this IAlgorithm algorithm, CancellationToken cancellationToken) {
      var executor = new AlgorithmExecutor(algorithm, cancellationToken);
      executor.StartSync();
    }
  }

  /// <summary>
  /// Can execute an algorithm synchronously
  /// </summary>
  internal class AlgorithmExecutor {
    private IAlgorithm algorithm;
    private AutoResetEvent waitHandle = new AutoResetEvent(false);
    private CancellationToken cancellationToken;

    public AlgorithmExecutor(IAlgorithm algorithm, CancellationToken cancellationToken) {
      this.algorithm = algorithm;
      this.cancellationToken = cancellationToken;
    }

    public void StartSync() {
      algorithm.Stopped += new EventHandler(algorithm_Stopped);
      algorithm.Paused += new EventHandler(algorithm_Paused);

      using (CancellationTokenRegistration registration = cancellationToken.Register(new Action(cancellationToken_Canceled))) {
        algorithm.Start();
        waitHandle.WaitOne();
        waitHandle.Dispose();
      }

      algorithm.Stopped -= new EventHandler(algorithm_Stopped);
      algorithm.Paused -= new EventHandler(algorithm_Paused);
      if (algorithm.ExecutionState == Core.ExecutionState.Started) {
        algorithm.Pause();
      }
      cancellationToken.ThrowIfCancellationRequested();
    }

    private void algorithm_Paused(object sender, EventArgs e) {
      waitHandle.Set();
    }

    private void algorithm_Stopped(object sender, EventArgs e) {
      waitHandle.Set();
    }

    private void cancellationToken_Canceled() {
      waitHandle.Set();
    }
  }
}
